import * as THREE from 'three';


class PanoramaViewer {
  image;
  camera;
  scene;
  renderer;
  mesh;
  sphere;

  fov = 90;
  minFov = 5;
  maxFov = 95;
  isUserInteracting = false;
  lon = 180;
  lat = 0;
  phi = 0;
  theta = 0;
  sphereRadius = 500;

  needsUpdate = false;

  constructor(container, settings={}, handleFovChange=null) {
    this.container = container;
    this.settings = settings;
    this.handleFovChange = handleFovChange;
    this.init();
  }

  init = () => {
    this.camera = new THREE.PerspectiveCamera(
      this.fov,
      this.container.offsetWidth / this.container.offsetHeight,
      1,
      this.sphereRadius
    );
    this.camera.target = new THREE.Vector3(0, 0, 0);

    this.scene = new THREE.Scene();

    this.addSphere();
    this.setRenderer();

    this.needsUpdate = true;
    this.animate();

    this.addEventListeners();
  }

  addSphere = () => {
    const geometry = new THREE.SphereGeometry(this.sphereRadius, 60, 40);
    this.material = new THREE.MeshBasicMaterial({ side: THREE.BackSide });
    this.sphere = new THREE.Mesh(geometry, this.material);
    this.sphere.scale.x = -1;

    if (this.settings.cameraYaw && !isNaN(this.settings.cameraYaw)) {
      this.setSphereYaw(this.settings.cameraYaw);
    }

    if (this.settings.cameraPitch && !isNaN(this.settings.cameraPitch)) {
      this.setSpherePitch(this.settings.cameraPitch);
    }

    if (this.settings.cameraRoll && !isNaN(this.settings.cameraRoll)) {
      this.setSphereRoll(this.settings.cameraRoll);
    }

    this.scene.add(this.sphere);
  }

  setRenderer = () => {
    this.renderer = new THREE.WebGLRenderer({
      preserveDrawingBuffer: true,
      alpha: true
    });

    this.renderer.setSize(this.container.offsetWidth, this.container.offsetHeight);
    this.container.appendChild(this.renderer.domElement);
  }

  addEventListeners = () => {
    this.container.addEventListener('mousedown', this.onMouseDown, false);
    this.container.addEventListener('mousemove', this.onMouseMove, false);
    this.container.addEventListener('mouseup', this.onMouseUp, false);
    this.container.addEventListener('mouseout', this.onMouseOut, false);
    this.container.addEventListener('mousewheel', this.onMouseWheel, false);
    this.container.addEventListener('DOMMouseScroll', this.onMouseWheel, false);
  }

  loadTexture = () => {
    const loader = new THREE.TextureLoader();

    return new Promise((resolve, reject) => {
      loader.load(this.image, (texture) => {
        // отключить ресайз при последующей загрузке в рендер
        texture.needsUpdate = true;
        texture.minFilter = THREE.LinearFilter;
        texture.generateMipmaps = false;

        resolve(texture);
      }, (err) => {
        reject(err);
      });
    });
  }

  onMouseDown = (event) => {
    this.isUserInteracting = true;

    this.container.classList.remove("cursor-grab");
    this.container.classList.add("cursor-grabbing");

    this.onPointerDownPointerX = event.clientX;
    this.onPointerDownPointerY = event.clientY;

    this.onPointerDownLon = this.lon;
    this.onPointerDownLat = this.lat;
  }

  onMouseMove = (event) => {
    if (this.isUserInteracting) {
      this.lon = (this.onPointerDownPointerX - event.clientX) * 0.1 + this.onPointerDownLon;
      this.lat = (event.clientY - this.onPointerDownPointerY) * 0.1 + this.onPointerDownLat;

      this.needsUpdate = true;
    }
  }

  onMouseUp = (event) => {
    this.isUserInteracting = false;

    this.container.classList.remove("cursor-grabbing");
    this.container.classList.add("cursor-grab");
  }

  onMouseOut = (event) => {
    this.isUserInteracting = false;

    this.container.classList.remove("cursor-grabbing");
    this.container.classList.add("cursor-grab");
  }

  onMouseWheel = (event) => {
    event.preventDefault();

    let cameraFov = this.camera.fov - event.wheelDeltaY * 0.05;
    if (cameraFov < this.minFov) {
      cameraFov = this.minFov;
    }
    if (cameraFov > this.maxFov) {
      cameraFov = this.maxFov;
    }

    this.setFov(cameraFov);

    this.needsUpdate = true;
  }

  animate = () => {
    if (this.needsUpdate) {
      this.render();
      this.needsUpdate = false;
    }

    requestAnimationFrame(this.animate);
  }

  render = () => {
    this.lat = Math.max(-85, Math.min(85, this.lat));
    this.phi = THREE.Math.degToRad(90 - this.lat);
    this.theta = THREE.Math.degToRad(this.lon);

    this.camera.target.x = this.sphereRadius * Math.sin(this.phi) * Math.cos(this.theta);
    this.camera.target.y = this.sphereRadius * Math.cos(this.phi);
    this.camera.target.z = this.sphereRadius * Math.sin(this.phi) * Math.sin(this.theta);

    this.camera.lookAt(this.camera.target);

    this.renderer.render(this.scene, this.camera);
  }

  cleanMaterial = () => {
    if (this.material) {
      if (this.material.map) this.material.map.dispose();
      if (this.material.lightMap) this.material.lightMap.dispose();
      if (this.material.bumpMap) this.material.bumpMap.dispose();
      if (this.material.normalMap) this.material.normalMap.dispose();
      if (this.material.specularMap) this.material.specularMap.dispose();
      if (this.material.envMap) this.material.envMap.dispose();

      this.material.dispose();
    }
  }

  setImage = async (image) => {
    this.cleanMaterial();
    this.image = image;
    const texture = await this.loadTexture();
    this.material.map = texture;

    this.needsUpdate = true;
  }

  setLatitude = (lat) => {
    this.lat = Math.max(-85, Math.min(85, lat));

    this.needsUpdate = true;
  }

  setLongitude = (lon) => {
    this.lon = lon;

    this.needsUpdate = true;
  }

  setFov = (fov) => {
    this.fov = Math.max(this.minFov, Math.min(this.maxFov, fov));
    this.camera.fov = this.fov;
    this.camera.updateProjectionMatrix();

    this.needsUpdate = true;
  }

  setSphereYaw = (yaw) => {
    yaw = Math.max(-180, Math.min(180, yaw));
    this.sphere.rotation.y = THREE.Math.degToRad(yaw);

    this.needsUpdate = true;
  }

  setSpherePitch = (pitch) => {
    pitch = Math.max(-90, Math.min(90, pitch));
    this.sphere.rotation.z = THREE.Math.degToRad(pitch);

    this.needsUpdate = true;
  }

  setSphereRoll = (roll) => {
    roll = Math.max(-90, Math.min(90, roll));
    this.sphere.rotation.x = THREE.Math.degToRad(roll);

    this.needsUpdate = true;
  }
}

export default PanoramaViewer;
