import React from 'react';
import { Button, ButtonGroup } from '@blueprintjs/core';

import PanoramaViewer from '../classes/PanoramaViewer';
import WindowSize from './WindowSize';


class Panorama extends React.Component {

  viewer;
  imageUrl;
  containerId = 'panorama-viewer';
  minZoom = 5;
  maxZoom = 95;
  canvasSizes = {
    "S": { width: 600, height: 300 },
    "M": { width: 800, height: 400 },
    "L": { width: 1000, height: 500 },
  };

  state = {
    zoom: this.minZoom,
    windowSize: "M"
  };

  componentDidMount = async () => {
    this.interval = setInterval(this.fetchQgisData, 100);

    if (!this.container) {
      this.setViewerContainer();
    }

    await this.setViewer();
  }

  componentDidUpdate = async (prevProps, prevState) => {
    if (!this.container) {
      this.setViewerContainer();
    }

    if (!this.viewer) {
      await this.setViewer();
      return;
    }

    if (this.state.windowSize != prevState.windowSize) {
      while (this.container.firstChild) {
          this.container.removeChild(this.container.lastChild);
      }
      await this.setViewer();
      return;
    }

    await this.updateViewer();
  }

  componentWillUnmount = () => {
    this.interval = null;
  }

  fetchQgisData = () => {
    const data = {
      imagePath: window.qgisApi.getImagePath(),
      windowSize: window.qgisApi.getWindowSize(),
    };

    this.setState({ ...data });
  }

  setViewerContainer = () => {
    this.container = document.getElementById(this.containerId);
  }

  setViewer = async () => {
    if (!this.container) {
      return;
    }

    const settings = {
      cameraYaw: window.qgisApi.getCameraYaw(),
      cameraPitch: window.qgisApi.getCameraPitch(),
      cameraRoll: window.qgisApi.getCameraRoll()
    };

    try {
      this.viewer = new PanoramaViewer(
        this.container,
        settings,
        this.handleFovChange
      );
      const imageUrl = this.getImageUrl();
      await this.viewer.setImage(imageUrl);
    } catch (error) {
      this.viewer = null;
      return;
    }
  }

  updateViewer = async () => {
    try {
      const imageUrl = this.getImageUrl();
      if (this.viewer.image !== imageUrl) {
        await this.viewer.setImage(imageUrl);
      }
    } catch (error) {
      this.viewer = null;
      return;
    }
  }

  getImageUrl = () => {
    if (!this.state.imagePath) {
      return null;
    }

    return "file:///" + this.state.imagePath;
  }

  handleFovChange = (value) => {
    let zoom = 100 - value;
    zoom = Math.max(this.minZoom, Math.min(this.maxZoom, zoom));
    this.setState({ zoom: zoom });
  }

  nextPoint = () => {
    window.qgisApi.selectNextFeature();
  }

  prevPoint = () => {
    window.qgisApi.selectPrevFeature();
  }

  firstPoint = () => {
    window.qgisApi.selectFirstFeature();
  }

  lastPoint = () => {
    window.qgisApi.selectLastFeature();
  }

  lookLeft = () => {
    if (this.viewer) {
      this.viewer.setLongitude(90);
      this.viewer.setLatitude(0);
    }
  }

  lookForward = () => {
    if (this.viewer) {
      this.viewer.setLongitude(180);
      this.viewer.setLatitude(0);
    }
  }

  lookBackward = () => {
    if (this.viewer) {
      this.viewer.setLongitude(0);
      this.viewer.setLatitude(0);
    }
  }

  lookRight = () => {
    if (this.viewer) {
      this.viewer.setLongitude(-90);
      this.viewer.setLatitude(0);
    }
  }

  render() {

    return (
      <React.Fragment>
        <div
          id={this.containerId}
          style={{
            width: this.canvasSizes[this.state.windowSize].width + "px",
            height: this.canvasSizes[this.state.windowSize].height + "px",
            marginBottom: "10px"
          }}
        />

        <div className="row">
          <div className="col-4">
            <ButtonGroup>
              <Button
                icon="chevron-backward"
                title="Начало"
                onClick={this.firstPoint}
              />
              <Button
                icon="chevron-left"
                title="Предыдyщая точка"
                onClick={this.prevPoint}
              />
              <Button
                icon="chevron-right"
                title="Следущая точка"
                onClick={this.nextPoint}
              />
              <Button
                icon="chevron-forward"
                title="Конец"
                onClick={this.lastPoint}
              />
            </ButtonGroup>
          </div>
          <div className="col-4">
            <ButtonGroup>
              <Button
                icon="arrow-left"
                onClick={this.lookLeft}
                title="Посмотреть налево"
              />
              <Button
                icon="arrow-up"
                onClick={this.lookForward}
                title="Посмотреть вперед"
              />
              <Button
                icon="arrow-down"
                onClick={this.lookBackward}
                title="Посмотреть назад"
              />
              <Button
                icon="arrow-right"
                onClick={this.lookRight}
                title="Посмотреть направо"
              />
            </ButtonGroup>
          </div>
          <div>
            <WindowSize windowSize={this.state.windowSize} />
          </div>
        </div>
      </React.Fragment>
    );
  }
}

export default Panorama;
