from qgis.PyQt import QtWebKitWidgets
from qgis.PyQt.QtNetwork import QNetworkProxyFactory
from qgis.PyQt.QtWebKit import QWebSettings
from qgis.PyQt.QtCore import QUrl, pyqtSlot, QVariant
from qgis.core import QgsExpression, QgsExpressionContextUtils


class ScolopaxWebView(QtWebKitWidgets.QWebView):

    WINDOW_SIZES = {
        'S': {
            'width': 600,
            'height': 430
        },
        'M': {
            'width': 800,
            'height': 570
        },
        'L': {
            'width': 1000,
            'height': 670
        }
    }

    def __init__(
        self,
        parent,
        layer,
        feature
    ):
        super().__init__()

        self.parent = parent
        self.layer = layer
        self.feature = feature

        self.init_window()
        self.set_web_settings()
        self.set_page()

    def set_page(self):
        self._page = self.page()
        self._page.mainFrame().javaScriptWindowObjectCleared.connect(
            self._populate_js_window_object
        )

    def _populate_js_window_object(self):
        self._page.mainFrame().addToJavaScriptWindowObject("qgisApi", self)

    def init_window(self):
        self.setWindowTitle("Scolopax QGIS")

        self.set_window_size(self.parent.webview_window_size)

        if self.parent.webview_position:
            self.move(self.parent.webview_position)

    def set_window_size(self, window_size):
        self.parent.set_webview_window_size(window_size)
        self.setFixedSize(
            self.WINDOW_SIZES[self.parent.webview_window_size]['width'],
            self.WINDOW_SIZES[self.parent.webview_window_size]['height']
        )

    def set_web_settings(self):
        QNetworkProxyFactory.setUseSystemConfiguration(True)

        settings = QWebSettings.globalSettings()
        settings.setAttribute(QWebSettings.PluginsEnabled, True)
        # settings.setAttribute(QWebSettings.DnsPrefetchEnabled, True)
        settings.setAttribute(QWebSettings.JavascriptEnabled, True)
        # settings.setAttribute(QWebSettings.JavascriptCanOpenWindows, True)
        # settings.setAttribute(QWebSettings.OfflineStorageDatabaseEnabled, True)
        # settings.setAttribute(QWebSettings.AutoLoadImages, True)
        # settings.setAttribute(QWebSettings.LocalStorageEnabled, True)
        # settings.setAttribute(QWebSettings.PrivateBrowsingEnabled, True)
        settings.setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        # settings.setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
        settings.setAttribute(QWebSettings.WebGLEnabled, True)
        settings.setAttribute(QWebSettings.AcceleratedCompositingEnabled, True)

    def show(self):
        url = QUrl.fromUserInput(f"file:///{self.parent.plugin_dir}/js/dist/index.html")
        self.load(url)
        super().show()

    def get_feature_attribute(self, name):
        value = self.feature.attributes()[self.feature.fieldNameIndex(name)]
        if type(value) == QVariant and QVariant.isNull(value):
            value = None

        return value

    def select_feature_by_index(self, index):
        self.layer.selectByExpression(f'"Index" = {index}')

        return self.layer.selectedFeatures()[0]

    @pyqtSlot(result=str)
    def getDriveName(self):
        return self.layer.name()

    @pyqtSlot(result=str)
    def getIndex(self):
        return str(self.get_feature_attribute('Index'))

    @pyqtSlot(result=str)
    def getImagePath(self):
        return str(self.get_feature_attribute('ImagePath'))

    @pyqtSlot(result=str)
    def getLat(self):
        return str(self.get_feature_attribute('Lat'))

    @pyqtSlot(result=str)
    def getLng(self):
        return str(self.get_feature_attribute('Lng'))

    @pyqtSlot(result=str)
    def getTime(self):
        return str(self.get_feature_attribute('Time'))

    @pyqtSlot(result=str)
    def getAddress(self):
        return str(self.get_feature_attribute('Address'))

    @pyqtSlot(result=str)
    def getDistance(self):
        return str(self.get_feature_attribute('Dist'))

    @pyqtSlot(result=str)
    def getRoadDistance(self):
        return str(self.get_feature_attribute('RDist'))

    @pyqtSlot(result=str)
    def getRoadSectionDistance(self):
        return str(self.get_feature_attribute('RSDist'))

    @pyqtSlot()
    def selectNextFeature(self):
        currentIndex = self.get_feature_attribute('Index')
        nextIndex = currentIndex + 1
        try:
            feature = self.select_feature_by_index(nextIndex)
            self.feature = feature
        except IndexError:
            return

    @pyqtSlot()
    def selectPrevFeature(self):
        currentIndex = self.get_feature_attribute('Index')
        prevIndex = currentIndex - 1
        try:
            feature = self.select_feature_by_index(prevIndex)
            self.feature = feature
        except IndexError:
            return

    @pyqtSlot()
    def selectFirstFeature(self):
        try:
            exp = QgsExpression(f"""
            aggregate(
                layer:='{self.layer.name()}',
                aggregate:='min',
                expression:=Index
            )""")
            firstIndex = exp.evaluate()

            feature = self.select_feature_by_index(firstIndex)
            self.feature = feature
        except Exception:
            return

    @pyqtSlot()
    def selectLastFeature(self):
        try:
            exp = QgsExpression(f"""
            aggregate(
                layer:='{self.layer.name()}',
                aggregate:='max',
                expression:=Index
            )""")
            firstIndex = exp.evaluate()

            feature = self.select_feature_by_index(firstIndex)
            self.feature = feature
        except Exception:
            return

    @pyqtSlot()
    def setWindowSmall(self):
        self.set_window_size("S")

    @pyqtSlot()
    def setWindowMedium(self):
        self.set_window_size("M")

    @pyqtSlot()
    def setWindowLarge(self):
        self.set_window_size("L")

    @pyqtSlot(result=str)
    def getWindowSize(self):
        return self.parent.webview_window_size

    @pyqtSlot(result=str)
    def getCameraYaw(self):
        return QgsExpressionContextUtils.layerScope(self.layer).variable('camera_yaw')

    @pyqtSlot(result=str)
    def getCameraPitch(self):
        return QgsExpressionContextUtils.layerScope(self.layer).variable('camera_pitch')

    @pyqtSlot(result=str)
    def getCameraRoll(self):
        return QgsExpressionContextUtils.layerScope(self.layer).variable('camera_roll')

    def moveEvent(self, e):
        self.parent.webview_position = self.pos()
        super().moveEvent(e)
